// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#include "IsoMeshSize.h"

#include <stdio.h>
#include <iostream>
#include <stdlib.h>
using std::string;

namespace MAd {

  // -------------------------------------------------------------------
  IsoMeshSize::IsoMeshSize():
    MeshSizeBase(), h(-1.)
  {}

  // -------------------------------------------------------------------
  IsoMeshSize::IsoMeshSize(double _h):
    MeshSizeBase()
  {
    h = _h;
  }

  // -------------------------------------------------------------------
  IsoMeshSize::IsoMeshSize(const IsoMeshSize *pm):
    MeshSizeBase()
  {
    h = pm->size();
  }

  // -------------------------------------------------------------------
  MeshSizeType IsoMeshSize::getType() const 
  {
    return ISOTROPIC;
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::intersect(const pMSize pMS0, const pMSize pMS1)
  {
    if ( pMS0->getType() != ISOTROPIC || 
         pMS1->getType() != ISOTROPIC    ) {
      printf("Error: intersecting anisotropic sizes in isotropic mesh size\n");
      exit(1);
    }

    double h0 = ((IsoMeshSize*)pMS0)->size();
    double h1 = ((IsoMeshSize*)pMS1)->size();
    h = std::min(h0,h1);
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::interpolate(const pMSize pMS0, const pMSize pMS1, 
                                double param)
  {
    if ( pMS0->getType() != ISOTROPIC || 
         pMS1->getType() != ISOTROPIC    ) {
      printf("Error: interpolating between anisotropic sizes in isotropic mesh size\n");
      exit(1);
    }

    double h0 = ((IsoMeshSize*)pMS0)->size();
    double h1 = ((IsoMeshSize*)pMS1)->size();
    h = h0 + param * ( h1 - h0 );
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::direction(int i, double dir[3]) const 
  {
    for (int c=0; c<3; c++) dir[c] = 0.;
    dir[i] = 1.;
  }

  // -------------------------------------------------------------------
  double IsoMeshSize::size(int i) const {
    return h;
  }

  // -------------------------------------------------------------------
  double IsoMeshSize::getMeanLength() const 
  {
    return h;
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::setSize(double _h)
  {
    h = _h;
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::scale(double factor)
  {
    h *= factor;
  }

  // -------------------------------------------------------------------
  double IsoMeshSize::lengthSqInDir(double vec[3]) const
  {
    return h*h;
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::toMetricSpace(const double vec[3], double tvec[3]) const
  {
    double invH = 1. / h;
    tvec[0] = vec[0] * invH;
    tvec[1] = vec[1] * invH;
    tvec[2] = vec[2] * invH;
  }

  // -------------------------------------------------------------------
  void IsoMeshSize::print(string name) const
  {
    std::cout<<"Printing IsoMeshSize \'"<<name<<"\' ("<<this<<")\n";
    std::cout<<"  h:\t"<<h<<"\n";
  }

  // -------------------------------------------------------------------

}
