// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Jean-Francois Remacle, Gaetan Compere
// -------------------------------------------------------------------

#ifdef _HAVE_GMSH_

#include "GmshModel.h"

#include "gmsh/discreteRegion.h"
#include "gmsh/discreteFace.h"
#include "gmsh/discreteEdge.h"
#include "gmsh/discreteVertex.h"

namespace MAd {

  // -------------------------------------------------------------------
  MAdGEntity * GmshModel::getEntityByTag(int dim, int tag)
  {
    switch(dim) {
    case 3: return getRegionByTag(tag);
    case 2: return getFaceByTag(tag);
    case 1: return getEdgeByTag(tag);
    case 0: return getVertexByTag(tag);
    }
  }

  // -------------------------------------------------------------------
  MAdGRegion * GmshModel::getRegionByTag(int tag)
  {
    GRegion * pGR = model->getRegionByTag(tag);
    if ( !pGR ) {
      pGR = new discreteRegion(model,tag);
      model->add(pGR);
    }
    return pGR;
  }

  // -------------------------------------------------------------------
  MAdGFace * GmshModel::getFaceByTag(int tag)
  {
    GFace * pGF = model->getFaceByTag(tag);
    if ( !pGF ) {
      pGF = new discreteFace(model,tag);
      model->add(pGF);
    }
    return pGF;
  }

  // -------------------------------------------------------------------
  MAdGEdge * GmshModel::getEdgeByTag(int tag)
  {
    GEdge * pGE = model->getEdgeByTag(tag);
    if ( !pGE ) {
      pGE = new discreteEdge(model,tag);
      model->add(pGE);
    }
    return pGE;
  }

  // -------------------------------------------------------------------
  MAdGVertex * GmshModel::getVertexByTag(int tag)
  {
    GVertex * pGV = model->getVertexByTag(tag);
    if ( !pGV ) {
      pGV = new discreteVertex(model,tag);
      model->add(pGV);
    }
    return pGV;
  }

  // -------------------------------------------------------------------
}

#endif
