// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Jean-Francois Remacle, Gaetan Compere
// -------------------------------------------------------------------

#ifndef _MESSAGE_H_
#define _MESSAGE_H_

#include <stdarg.h>
#include <string>

#define MDB_FATAL          1  // Fatal error (causes Gmsh to exit)
#define MDB_FATAL1         2  // First part of a multiline FATAL message 
#define MDB_FATAL2         3  // Middle part of a multiline FATAL message
#define MDB_FATAL3         4  // Last part of a multiline FATAL message  

#define MDB_GERROR         5  // Error (but Gmsh can live with it)
#define MDB_GERROR1        6  // First part of a multiline ERROR message 
#define MDB_GERROR2        7  // Middle part of a multiline ERROR message
#define MDB_GERROR3        8  // Last part of a multiline ERROR message  

#define MDB_WARNING        9  // Warning
#define MDB_WARNING1      10  // First part of a multiline WARNING message 
#define MDB_WARNING2      11  // Middle part of a multiline WARNING message
#define MDB_WARNING3      12  // Last part of a multiline WARNING message  

#define MDB_INFO          13  // Long informations
#define MDB_INFO1         14  // First part of a multiline INFO message 
#define MDB_INFO2         15  // Middle part of a multiline INFO message
#define MDB_INFO3         16  // Last part of a multiline INFO message  

#define MDB_STATUS1       21  // Small information in status bar (left)
#define MDB_STATUS2       22  // Small interaction in status bar (middle)
#define MDB_STATUS3       23  // Small interaction in status bar (right)

#define MDB_STATUS1N      24  // Same as STATUS1, but not going into the log file
#define MDB_STATUS2N      25  // Same as STATUS2, but not going into the log file
#define MDB_STATUS3N      26  // Same as STATUS3, but not going into the log file

#define MDB_ONSCREEN      27  // Persistent on-screen message

#define MDB_DIRECT        30  // Direct message (no special formatting)
#define MDB_SOLVER        31  // Solver message
#define MDB_SOLVERR       32  // Solver errors and warnings

#define MDB_PROGRESS      40  // Progress indicator

#define MDB_WHITE_STR          "        : "
#define MDB_FATAL_STR          "Fatal   : "
#define MDB_ERROR_STR          "Error   : "
#define MDB_WARNING_STR        "Warning : "
#define MDB_INFO_STR           "Info    : "
/* #define MDB_DEBUG_STR          "Debug   : " */
#define MDB_STATUS_STR         "Info    : "

namespace MAd {

  void   Signal(int signum);
  void   Msg(int level, std::string fmt, ...);
  void   Msg_char(int level, const char * fmt, ...);
  double Cpu(void);
  double GetValue(char *text, double defaultval);

}

#endif
